#!/usr/bin/env python3

import os
import sys
import logging
import yaml
from schema import Schema, And, Or, Use, Optional, SchemaError


DEFAULT = {
    'log': {
        'disable_existing_loggers': False,
        'version': 1,
        'formatters': {
            'short': {
                'format': '%(asctime)s %(levelname)s %(module)s: %(message)s'
            }
        },
        'handlers': {
            'console': {
                'level': 'DEBUG',
                'formatter': 'short',
                'class': 'logging.StreamHandler'
            }
        },
        'loggers': {
            '': {
                'handlers': ['console'],
                'level': 'DEBUG'
            }
        }
    }
}


def port_range(port):
    return 0 <= port <= 65535


schema = Schema({
    'zmq': {
        'host': And(str, len),
        'port': And(Use(int), port_range),
        'timeout': And(Use(int), lambda x: x > 0)
    },
    'azure': {
        'connection_string': And(str, len)
    },
    Optional('log'): dict
})


def load_config(config_file):
    config = yaml.safe_load(config_file)
    try:
        config = schema.validate(config)
    except SchemaError as e:
        # Better error format
        error = str(e).splitlines()
        del error[1]
        raise Exception(' '.join(error))

    _apply_default(config, DEFAULT)

    return config


def _apply_default(config, default):
    for key in default:
        if isinstance(default[key], dict):
            if key not in config:
                config[key] = default[key]
            else:
                _apply_default(config[key], default[key])
        else:
            config[key] = default[key]



import iothub_client
import logging

PROTOCOL = iothub_client.IoTHubTransportProvider.AMQP
RECEIVE_CONTEXT = 0
CONNECTION_STATUS_CONTEXT = 0
MESSAGE_TIMEOUT = 10000


class AzureClient:

    def __init__(self, connection_string):
        self._client = iothub_client.IoTHubClient(connection_string, PROTOCOL)
        self._client.set_option('messageTimeout', MESSAGE_TIMEOUT)
        self._client.set_connection_status_callback(self._connection_status_callback, CONNECTION_STATUS_CONTEXT)
        self._client.set_message_callback(self._receive_message_callback, RECEIVE_CONTEXT)

    def send(self, message):
        try:
            m = iothub_client.IoTHubMessage(message)
            self._client.send_event_async(m, self._send_confirmation_callback, None)
        except iothub_client.IoTHubError as e:
            logging.error('Azure IoT Hub error: %s', e)
        except Exception as e:
            logging.error('Unhandled exception', exc_info=True)

    def _connection_status_callback(self, result, reason, user_context):
        logging.info('Connection status: %s (%d)', result, reason)

    def _receive_message_callback(self, message, counter):
        logging.info('Received message: %s', message)

    def _send_confirmation_callback(self, message, result, user_context):
        logging.info('Send confirmation: %s', result)



import click
import json
import logging.config
import sys
import yaml
import zmq
from . import azure
from .config import load_config

__version__ = '1.1.1'


@click.command()
@click.option('--config', '-c', 'config_file', type=click.File('r'), required=True, help='Configuration file.')
@click.option('--test', is_flag=True, help='Test configuration file.')
@click.version_option(version=__version__)
def cli(config_file, test=False):
    '''ZeroMQ to Azure IoT Hub.'''

    try:
        config = load_config(config_file)
        config_file.close()
    except Exception as e:
        logging.error('Failed opening configuration file')
        logging.error(str(e))
        sys.exit(1)

    if test:
        click.echo("The configuration file seems ok")
        return

    logging.config.dictConfig(config['log'])
    logging.info('Process started')

    server(config)


def server(config):
    context = zmq.Context()

    sock = context.socket(zmq.SUB)
    sock.setsockopt_string(zmq.SUBSCRIBE, '')
    '''
    sock.setsockopt(zmq.RCVTIMEO, config['zmq']['timeout'])
    '''
    sock.connect('tcp://%s:%d' % (config['zmq']['host'], config['zmq']['port']))

    try:
        az = azure.AzureClient(config['azure']['connection_string'])
    except Exception as e:
        logging.error('Failed to create Azure IoT Hub client', exc_info=True)
        sys.exit(1)

    while True:
        try:
            message = sock.recv_json()
            az.send(json.dumps(message))
        except zmq.error.Again as e:
            logging.error('ZeroMQ error: %s' % e)
        except Exception:
            logging.error('Unhandled exception', exc_info=True)


def main():
    try:
        cli()
    except KeyboardInterrupt:
        pass
    except Exception as e:
        click.echo(str(e), err=True)
        sys.exit(1)


if __name__ == '__main__':
    main()



from setuptools import setup

setup(
    name='cp2azure',
    packages=['cp2azure'],
    version='1.1.1',
    description='COOPER to Azure IoT Hub',
    url='https://github.com/hardwario/cp2azure',
    author='HARDWARIO s.r.o.',
    author_email='ask@hardwario.com',
    license='MIT',
    keywords = ['cooper', 'azure', 'iot'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Topic :: Scientific/Engineering :: Human Machine Interfaces',
        'Environment :: Console',
        'Intended Audience :: Science/Research'
    ],
    install_requires=[
        'azure-iothub-device-client==1.4.3', 'click==6.7', 'PyYAML==3.13','pyzmq==17.1.2','schema'
    ],
    entry_points='''
        [console_scripts]
        cp2azure=cp2azure.app:main
    '''
)
